﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Interfaces;
using System.ComponentModel;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.CAG.MVPView
{
	/// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
	/// Property Changed Delegate
	/// </summary>
	public delegate void AllPropertyChangedDelegate(object sender, string propertyName);

	/// <summary>
	/// View ViewModel
	/// </summary>
	public class MVPViewModel : FrameworkElement, IMVPViewModel, INotifyPropertyChanged
	{
		/// <summary>
		/// Occurs when [on all property changed events].
		/// </summary>
		public event AllPropertyChangedDelegate OnAllPropertyChangedEvents = delegate { };

		/// <summary>
		/// Occurs when a property value changes.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged = delegate { };

		#region PROPERTY: Error
		/// <summary>
		/// Gets or sets the error.
		/// </summary>
		/// <value>The error.</value>
		[Dependency]
		public IError Error { get; set; }

		#endregion
		#region PROPERTY: Logger
		/// <summary>
		/// Gets or sets the logger.
		/// </summary>
		/// <value>The logger.</value>
		[Dependency]
		public ILoggerFacade Logger { get; set; }

		#endregion

		#region PROPERTY: SysConfig w/virtual set
		private ISystemConfiguration _sysConfig;
		/// <summary>
		/// Gets or sets the sys config.
		/// </summary>
		/// <value>The sys config.</value>
		[Dependency]
		public ISystemConfiguration SysConfig
		{
			get { return _sysConfig; }
			set
			{
				_sysConfig = value;
				OnSysConfigSet();
			}
		}
		/// <summary>
		/// Called when [sys config set].
		/// </summary>
		protected virtual void OnSysConfigSet()
		{
		}

		#endregion
		#region PROPERTY: Container w/virtual set
		private IUnityContainer _Container;
		/// <summary>
		/// Gets or sets the Container
		/// </summary>
		/// <value></value>
		[Dependency]
		public IUnityContainer Container
		{
			get { return _Container; }
			set
			{
				_Container = value;
				OnContainerSet();
			}
		}
		/// <summary>
		/// Intended to be overridden 
		/// </summary>
		protected virtual void OnContainerSet()
		{
		}
		#endregion
		#region PROPERTY: Presenter w/virtual set
		private IMVPPresenter _Presenter;
		/// <summary>
		/// Gets or sets the Presenter
		/// </summary>
		/// <value>The Presenter</value>
		public IMVPPresenter Presenter
		{
			get { return _Presenter; }
			set
			{
				_Presenter = value;
				OnPresenterSet();
			}
		}
		/// <summary>
		/// Intended to be overridden 
		/// </summary>
		protected virtual void OnPresenterSet()
		{
		}
		#endregion

		/// <summary>
		/// Initializes a new instance of the <see cref="MVPViewModel"/> class.
		/// </summary>
		/// <param name="container">The container.</param>
		public MVPViewModel(IUnityContainer container)
		{
			container.BuildUp(this);
			container.BuildUp(GetType(), this);
		}

		/// <summary>
		/// Called when [property changed].
		/// </summary>
		/// <param name="propertyName">Name of the property.</param>
		public void OnNotifyPropertyChange(string propertyName)
		{
			PropertyChanged(this, new PropertyChangedEventArgs(propertyName));

			OnAllPropertyChangedEvents(GetType(), propertyName);
		}
	}
}
