﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.ComponentModel.Composition.Primitives;

namespace GWN.Contrib.MEF.Base
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// MEF Bootstrapper 
    /// </summary>
    public class MEFBootstrapper : CompositionContainer
    {
        private object parent;
        /// <summary>
        /// Initializes a new instance of the <see cref="MEFBootstrapper"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        public MEFBootstrapper(object parent)
        {
            this.parent = parent;
        }

        /// <summary>
        /// Returns all exports that match the conditions of the specified import.
        /// </summary>
        /// <param name="definition">The <see cref="ImportDefinition"/> that defines the conditions of the
        /// <see cref="Export"/> to get.</param>
        /// <param name="atomicComposition"></param>
        /// <returns></returns>
        /// <result>
        /// An <see cref="IEnumerable{T}"/> of <see cref="Export"/> objects that match
        /// the conditions defined by <see cref="ImportDefinition"/>, if found; otherwise, an
        /// empty <see cref="IEnumerable{T}"/>.
        /// </result>
        /// <remarks>
        /// 	<note type="inheritinfo">
        /// The implementers should not treat the cardinality-related mismatches as errors, and are not
        /// expected to throw exceptions in those cases.
        /// For instance, if the import requests exactly one export and the provider has no matching exports or more than one,
        /// it should return an empty <see cref="IEnumerable{T}"/> of <see cref="Export"/>.
        /// </note>
        /// </remarks>
        protected override IEnumerable<Export> GetExportsCore(ImportDefinition definition, AtomicComposition atomicComposition)
        {
            return base.GetExportsCore(definition, atomicComposition);
        }

        /// <summary>
        /// Raises the <see cref="ExportsChanged"/> event.
        /// </summary>
        /// <param name="e">An <see cref="ExportsChangeEventArgs"/> containing the data for the event.</param>
        protected override void OnExportsChanged(ExportsChangeEventArgs e)
        {
            base.OnExportsChanged(e);
        }

        /// <summary>
        /// Raises the <see cref="ExportsChanging"/> event.
        /// </summary>
        /// <param name="e">An <see cref="ExportsChangeEventArgs"/> containing the data for the event.</param>
        protected override void OnExportsChanging(ExportsChangeEventArgs e)
        {
            base.OnExportsChanging(e);
        }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        public void Initialize()
        {
            //var catalog = new AggregateCatalog();
            //catalog.Catalogs.Add(new AssemblyCatalog(parent.GetType().Assembly));
            //catalog.Catalogs.Add(new DirectoryCatalog("AddIn"));

            //CompositionBatch batch = new CompositionBatch();
            //batch.AddPart(parent);
            //batch.AddExportedValue<CompositionContainer>(this);

            //try
            //{
            //    Compose(batch);
            //}
            //catch (Exception ex)
            //{
            //    throw;
            //}
        }
    }
}
