﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity.InterceptionExtension;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Interfaces;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.MVC2.CallHandlers
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// Controller Matching Rule
    /// </summary>
    public class ControllerMatchingRule : IMatchingRule
    {
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }

        /// <summary>
        /// Tests to see if this rule applies to the given member.
        /// </summary>
        /// <param name="member">Member to test.</param>
        /// <returns>
        /// true if the rule applies, false if it doesn't.
        /// </returns>
        public bool Matches(System.Reflection.MethodBase member)
        {
            // We're only interested in IControllerFactory (not 
            // the DefaultControllerFactory which will also pass through here)
            if (!member.DeclaringType.FullName.Contains("IControllerFactory"))
                return false;

            Logger.Log(string.Format("ControllerMatchingRule.Matches(member=[{0}])",
                member.DeclaringType.FullName),
                Category.Debug, Priority.None);

            return true;
        }

    }
}
