﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Web.Routing;
using Microsoft.Practices.Unity;
using GWN.Contrib.MVC2.Interfaces;
using GWN.Contrib.Unity.Interfaces;
using Microsoft.Practices.ObjectBuilder2;
using Microsoft.Practices.Composite.UnityExtensions;
using System.Diagnostics;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.MVC2.Factories
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// Custom Controller Factory
    /// </summary>
    public class CustomControllerFactory : IControllerFactory
    {
        /// <summary>
        /// Gets or sets the error.
        /// </summary>
        /// <value>The error.</value>
        [Dependency]
        public IError Error { get; set; }

        /// <summary>
        /// Gets or sets the container.
        /// </summary>
        /// <value>The container.</value>
        [Dependency]
        public IUnityContainer Container { get; set; }

        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="CustomControllerFactory"/> class.
        /// </summary>
        public CustomControllerFactory()
        {

        }


        /// <summary>
        /// Creates the controller.
        /// </summary>
        /// <param name="requestContext">The request context.</param>
        /// <param name="controllerName">Name of the controller.</param>
        /// <returns>The controller.</returns>
        public IController CreateController(RequestContext requestContext, string controllerName)
        {
            if (controllerName.Contains("."))
                return null;


            Logger.Log("BEFORE CREATE:: CustomControllerFactory.CreateController()",
                Category.Debug, Priority.None);


            // IControllerFactory without name will return the original 
            // ControllerFactory.  We're going going to let it do it's job
            // and get a reference to the controller.
            IControllerFactory factory = Container.Resolve<IControllerFactory>();

            IController controller = null;
            try
            {
                controller = factory.CreateController(requestContext, controllerName);
            }
            catch (Exception ex) // If controllerName is not supported do not crash application
            {
                string errMsg = null;

                // Let user know if we couldn't find hte silverlight file
                if (ex.Message.Contains(".xap") && ex.Message.Contains("controller for path"))
                    errMsg = string.Format(
                        "{0} was not found in ClientBin!  Check your .ASPX Source setting.  Reported error follows:",
                        ex.Message.Split(' ').FirstOrDefault<string>(s => s.Contains(".xap")));

                else // report error as applicable
                    errMsg = string.Format("{0} could not create {1}", GetType().FullName, controllerName);

                Error.SetError(ex, errMsg);
            }

            if (controller != null)
                Logger.Log(string.Format("AFTER CREATE:: CustomControllerFactory.CreateController() CREATED [{0}]",
                    controller.GetType().Name), Category.Debug, Priority.None);

            return controller;
        }

        /// <summary>
        /// Releases the controller.
        /// </summary>
        /// <param name="controller">The controller.</param>
        public void ReleaseController(IController controller)
        {
            if (controller is IControllerPrerelease)
                ((IControllerPrerelease)controller).OnPreControllerRelease();

            Container.Resolve<IControllerFactory>()
                .ReleaseController(controller);

            Logger.Log(string.Format("CustomControllerFactory.ReleaseController() RELEASED [{0}]",
                controller.GetType().Name), Category.Debug, Priority.None);
        }
    }
}
