﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Menu;
using System.Collections.Specialized;
using System.Configuration;
using System.Runtime.Serialization;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Entities;

namespace GWN.Contrib.Unity.Configuration
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// System Configuration 
    /// </summary>
    [Serializable]
    public class DefaultSysConfig : ISystemConfiguration
    {

        /// <summary>
        /// Unity Container available as global resources
        /// </summary>
        public static IUnityContainer _container;

        /// <summary>
        /// Gets or sets the container.
        /// </summary>
        /// <value>The container.</value>
        [Dependency]
        public static IUnityContainer Container
        {
            get { return _container; }
            set { _container = value; }
        }

        /// <summary>
        /// Gets or sets the WPF menus.
        /// </summary>
        /// <value>The WPF menus.</value>
        public List<MenuItem> WPFMenus { get; set; }

        /// <summary>
        /// Gets or sets the website menus.
        /// </summary>
        /// <value>The website menus.</value>
        public List<MenuItem> WebsiteMenus { get; set; }

        /// <summary>
        /// Gets or sets the app settings.
        /// </summary>
        /// <value>The app settings.</value>
        public List<KeyValueData> AppSettings { get; set; }

        /// <summary>
        /// Gets or sets the connection strings.
        /// </summary>
        /// <value>The connection strings.</value>
        public List<KeyValueData> ConnectionStrings { get; set; }

        /// <summary>
        /// Gets or sets the version.
        /// </summary>
        /// <value>The version.</value>
        public string Version
        {
            get
            {
                return GetAppSetting("Version", "1.0.0.0");
            }
        }

        /// <summary>
        /// Gets the app setting specified by key
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        public string GetAppSetting(string key)
        {
            return GetAppSetting(key, "");
        }

        /// <summary>
        /// Gets the app setting specified by key.  If not found
        /// it returns defaultValue
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <returns></returns>
        public string GetAppSetting(string key, string defaultValue)
        {
            KeyValueData result = AppSettings
                .FirstOrDefault<KeyValueData>(f => f.Key.ToLower().Equals(key.ToLower()));

            if (result == null)
                return defaultValue;

            return result.Value;
        }


        /// <summary>
        /// Initializes a new instance of the <see cref="DefaultSysConfig"/> class.
        /// </summary>
        public DefaultSysConfig()
        {
            AppSettings = new List<KeyValueData>();
            ConnectionStrings = new List<KeyValueData>();
        }


    }
}
