﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GWN.Contrib.Unity.Interfaces;
using System.Diagnostics;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.Unity.Errors
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0 
    /// </remarks>
    /// Default error object
    /// </summary>
    public class DefaultError : IError
    {
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is error.
        /// </summary>
        /// <value><c>true</c> if this instance is error; otherwise, <c>false</c>.</value>
        public bool IsError { get; private set; }

        private Exception _error;
        /// <summary>
        /// Gets or sets the error.
        /// </summary>
        /// <value>The error.</value>
        public Exception Error
        {
            get { return _error; }
            set
            {
                _error = value;
                IsError = true;
            }
        }

        /// <summary>
        /// Gets or sets the errors.  Assumes IError is registered
        /// as a singleton otherwise contains only current error
        /// </summary>
        /// <value>The errors.</value>
        public List<Exception> Errors { get; private set; }

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <returns></returns>
        public bool SetError(Exception ex)
        {
            return SetError(ex, "");
        }

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <param name="friendlyMessage">The friendly message.</param>
        /// <returns></returns>
        public bool SetError(Exception ex, string friendlyMessage)
        {
            return SetError(ex, friendlyMessage, "");
        }

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <param name="friendlyMessage">The friendly message.</param>
        /// <param name="source">The source.</param>
        /// <returns></returns>
        public bool SetError(Exception ex, string friendlyMessage, string source)
        {
            IsError = true;
            Error = ex;
#if SILVERLIGHT
			Error.Data.Add("friendlyMessage", friendlyMessage);
			Error.Data.Add("source", source);
#else
            Error.HelpLink = friendlyMessage;
            Error.Source = source;
#endif

            // Add the error to list (assuming singleton configuration)
            if (Errors == null)
                Errors = new List<Exception>();

            Errors.Add(Error);

#if SILVERLIGHT
			string message = string.Format("{0}{1}", 
				string.IsNullOrEmpty(friendlyMessage) ? "" : friendlyMessage+"\r\n",
					ex.Message);
#else
            string message = string.Format("{0}{1}{2}",
                string.IsNullOrEmpty(friendlyMessage) ? "" : friendlyMessage + "\r\n",
                Error.Source == null ? "" : "Source: " + Error.Source + "\r\n",
                ex.Message);
#endif
            if (Logger != null)
                Logger.Log(string.Format("ERROR: {0} {1}",
                    message, ex.StackTrace), Category.Exception, Priority.High);


            return false; // permits code to set error and return status on one line
        }

        /// <summary>
        /// Clears the errors.
        /// </summary>
        public void ClearErrors()
        {
            Errors.Clear();
            IsError = false;
        }


        /// <summary>
        /// Gets the friendly message.
        /// </summary>
        /// <value>The friendly message.</value>
        public string FriendlyMessage
        {
            get
            {
                if (Error == null)
                    return "";

#if SILVERLIGHT
				if (Error.Data.Count > 0)
				{
					var data = Error.Data["friendlyMessage"];
					if (data != null)
						return data.ToString();
				}
				return "";
			}

#else
                return Error == null ? "" : Error.HelpLink;
            }
#endif

        }
    }
}
