﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Entities;
using GWN.Contrib.Unity.Configuration;

namespace GWN.Contrib.Unity.Extensions
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0 
    /// </remarks>
    /// Object Extension
    /// </summary>
    public static class ObjectExtension
    {
        /// <summary>
        /// Gets the error.
        /// </summary>
        /// <value>The error.</value>
        public static IError Error { get { return DefaultSysConfig.Container.Resolve<IError>(); } }

        /// <summary>
        /// Gets the container.
        /// </summary>
        /// <value>The container.</value>
        public static IUnityContainer Container { get { return DefaultSysConfig.Container; } }

        /// <summary>
        /// Serializes the object.  
        /// <example>
        /// string xml = ((object)myObj).SerializeObject();
        /// </example>
        /// </summary>
        /// <param name="objToSerialize">The p object.</param>
        /// <returns></returns>
        public static String SerializeObject(this object objToSerialize)
        {
            try
            {
                String XmlizedString = null;
                MemoryStream memoryStream = new MemoryStream();
                XmlSerializer xs = new XmlSerializer(objToSerialize.GetType());
                StreamWriter xmlTextWriter = new StreamWriter(memoryStream, Encoding.UTF8);
                xs.Serialize(xmlTextWriter, objToSerialize);
                memoryStream = (MemoryStream)xmlTextWriter.BaseStream;
                XmlizedString = ByteHelper.Instance.UTF8ByteArrayToString(memoryStream.ToArray());
                return XmlizedString;
            }
            catch (Exception ex)
            {
                Error.SetError(ex);
                return null;
            }
        }

        /// <summary>
        /// Deserializes the object.  
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="pObject">The p object.</param>
        /// <param name="pXmlizedString">The p xmlized string.</param>
        /// <returns></returns>
        public static T DeserializeObject<T>(this object pObject, String pXmlizedString)
        {
            try
            {
                XmlSerializer xs = new XmlSerializer(pObject.GetType());

                MemoryStream memoryStream = new MemoryStream(
                    ByteHelper.Instance.StringToUTF8ByteArray(pXmlizedString));

                StreamWriter xmlTextWriter = new StreamWriter(memoryStream, Encoding.UTF8);
                return (T)xs.Deserialize(memoryStream);
            }
            catch (Exception ex)
            {
                Error.SetError(ex);
                return default(T);
            }
        }


    }
}
