﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GWN.Contrib.Unity.Interfaces
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// Error interface
    /// </summary>
    public interface IError
    {
        /// <summary>
        /// Gets a value indicating whether this instance is error.
        /// </summary>
        /// <value><c>true</c> if this instance is error; otherwise, <c>false</c>.</value>
        bool IsError { get; }

        /// <summary>
        /// Gets the error.
        /// </summary>
        /// <value>The error.</value>
        Exception Error { get; }

        /// <summary>
        /// Gets the errors.
        /// </summary>
        /// <value>The errors.</value>
        List<Exception> Errors { get; }

        /// <summary>
        /// Gets the friendly message.
        /// </summary>
        /// <value>The friendly message.</value>
        string FriendlyMessage { get; }

        /// <summary>
        /// Clears the errors.
        /// </summary>
        void ClearErrors();

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <returns></returns>
        bool SetError(Exception ex);

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <param name="friendlyMessage">The friendly message.</param>
        /// <returns></returns>
        bool SetError(Exception ex, string friendlyMessage);

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <param name="friendlyMessage">The friendly message.</param>
        /// <param name="source">The source.</param>
        /// <returns></returns>
        bool SetError(Exception ex, string friendlyMessage, string source);
    }
}
