﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;

namespace GWN.Contrib.Unity.Menu
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// 
    /// </summary>
    public class MenuElement : ConfigurationElement
    {
        /// <summary>
        /// Gets or sets a value indicating whether this instance is active.
        /// </summary>
        /// <value><c>true</c> if this instance is active; otherwise, <c>false</c>.</value>
        [ConfigurationProperty("isActive", DefaultValue = true, IsRequired = false)]
        public bool IsActive
        {
            get { return (bool)this["isActive"]; }
            set { this["isActive"] = value; }
        }

        /// <summary>
        /// Gets or sets the menu.
        /// </summary>
        /// <value>The menu.</value>
        [ConfigurationProperty("menu", DefaultValue = "main", IsRequired = false)]
        public string Menu
        {
            get { return (string)this["menu"]; }
            set { this["menu"] = value; }
        }

        /// <summary>
        /// Gets or sets the menu parent.
        /// </summary>
        /// <value>The menu parent.</value>
        [ConfigurationProperty("menuParent", DefaultValue = "root", IsRequired = false)]
        public string MenuParent
        {
            get { return (string)this["menuParent"]; }
            set { this["menuParent"] = value; }
        }

        /// <summary>
        /// Gets or sets the link text.
        /// </summary>
        /// <value>The link text.</value>
        [ConfigurationProperty("linkText", IsRequired = true)]
        public string LinkText
        {
            get { return (string)this["linkText"]; }
            set { this["linkText"] = value; }
        }

        /// <summary>
        /// Gets or sets the name of the action.
        /// </summary>
        /// <value>The name of the action.</value>
        [ConfigurationProperty("actionName", DefaultValue = "index", IsRequired = true)]
        public string ActionName
        {
            get { return (string)this["actionName"]; }
            set { this["actionName"] = value; }
        }

        /// <summary>
        /// Gets or sets the name of the controller.
        /// </summary>
        /// <value>The name of the controller.</value>
        [ConfigurationProperty("controllerName", DefaultValue = "home", IsRequired = true)]
        public string ControllerName
        {
            get { return (string)this["controllerName"]; }
            set { this["controllerName"] = value; }
        }

        /// <summary>
        /// Gets or sets the roles.
        /// </summary>
        /// <value>The roles.</value>
        [ConfigurationProperty("roles", DefaultValue = "*", IsRequired = false)]
        public string Roles
        {
            get { return (string)this["roles"]; }
            set { this["roles"] = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MenuElement"/> class.
        /// </summary>
        /// <param name="elementName">Name of the element.</param>
        public MenuElement(string elementName)
        {
            LinkText = elementName;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MenuElement"/> class.
        /// </summary>
        /// <param name="menuParent">The menu parent.</param>
        /// <param name="menu">The menu.</param>
        /// <param name="linkText">The link text.</param>
        /// <param name="actionName">Name of the action.</param>
        /// <param name="controllerName">Name of the controller.</param>
        /// <param name="roles">The roles.</param>
        /// <param name="isActive">if set to <c>true</c> [is active].</param>
        public MenuElement(
            string menuParent,
            string menu,
            string linkText,
            string actionName,
            string controllerName,
            string roles,
            bool isActive)
        {
            IsActive = isActive;
            Menu = menu;
            MenuParent = menuParent;
            LinkText = linkText;
            ActionName = actionName;
            ControllerName = controllerName;
            Roles = roles;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MenuElement"/> class.
        /// </summary>
        public MenuElement() { }



    }
}
