﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;

namespace GWN.Contrib.Unity.Menu
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// Menu Element Collection
    /// </summary>
    public class MenuElementCollection : ConfigurationElementCollection
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MenuElementCollection"/> class.
        /// </summary>
        public MenuElementCollection()
        {
        }

        /// <summary>
        /// Gets the menu collection.
        /// </summary>
        /// <returns></returns>
        public List<MenuItem> Items
        {
            get
            {
                List<MenuItem> collection = new List<MenuItem>();
                for (int i = 0; i < Count; i++)
                {
                    collection.Add(new MenuItem
                    {
                        IsActive = this[i].IsActive,
                        Menu = this[i].Menu,
                        MenuParent = this[i].MenuParent,
                        LinkText = this[i].LinkText,
                        ActionName = this[i].ActionName,
                        ControllerName = this[i].ControllerName,
                        Roles = this[i].Roles
                    });
                }
                return collection;
            }
        }


        /// <summary>
        /// Gets the type of the <see cref="T:System.Configuration.ConfigurationElementCollection"/>.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The <see cref="T:System.Configuration.ConfigurationElementCollectionType"/> of this collection.
        /// </returns>
        public override ConfigurationElementCollectionType CollectionType
        {
            get { return ConfigurationElementCollectionType.AddRemoveClearMap; }
        }

        /// <summary>
        /// When overridden in a derived class, creates a new <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </summary>
        /// <returns>
        /// A new <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new MenuElement();
        }

        /// <summary>
        /// Creates a new <see cref="T:System.Configuration.ConfigurationElement"/> when overridden in a derived class.
        /// </summary>
        /// <param name="elementName">The name of the <see cref="T:System.Configuration.ConfigurationElement"/> to create.</param>
        /// <returns>
        /// A new <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </returns>
        protected override ConfigurationElement CreateNewElement(string elementName)
        {
            return new MenuElement(elementName);
        }

        /// <summary>
        /// Gets the element key for a specified configuration element when overridden in a derived class.
        /// </summary>
        /// <param name="element">The <see cref="T:System.Configuration.ConfigurationElement"/> to return the key for.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> that acts as the key for the specified <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((MenuElement)element).LinkText;
        }


        /// <summary>
        /// Gets or sets the name of the <see cref="T:System.Configuration.ConfigurationElement"/> to associate with the add operation in the <see cref="T:System.Configuration.ConfigurationElementCollection"/> when overridden in a derived class.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The name of the element.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// The selected value starts with the reserved prefix "config" or "lock".
        /// </exception>
        public new string AddElementName
        {
            get
            { return base.AddElementName; }

            set
            { base.AddElementName = value; }
        }

        /// <summary>
        /// Gets or sets the name for the <see cref="T:System.Configuration.ConfigurationElement"/> to associate with the clear operation in the <see cref="T:System.Configuration.ConfigurationElementCollection"/> when overridden in a derived class.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The name of the element.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// The selected value starts with the reserved prefix "config" or "lock".
        /// </exception>
        public new string ClearElementName
        {
            get
            { return base.ClearElementName; }

            set
            { base.AddElementName = value; }
        }

        /// <summary>
        /// Gets or sets the name of the <see cref="T:System.Configuration.ConfigurationElement"/> to associate with the remove operation in the <see cref="T:System.Configuration.ConfigurationElementCollection"/> when overridden in a derived class.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The name of the element.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// The selected value starts with the reserved prefix "config" or "lock".
        /// </exception>
        public new string RemoveElementName
        {
            get
            { return base.RemoveElementName; }
        }

        /// <summary>
        /// Gets the number of elements in the collection.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The number of elements in the collection.
        /// </returns>
        public new int Count
        {
            get { return base.Count; }
        }


        /// <summary>
        /// Gets or sets the <see cref="GWN.Contrib.Unity.Menu.MenuElement"/> at the specified index.
        /// </summary>
        /// <value></value>
        public MenuElement this[int index]
        {
            get
            {
                return (MenuElement)BaseGet(index);
            }
            set
            {
                if (BaseGet(index) != null)
                {
                    BaseRemoveAt(index);
                }
                BaseAdd(index, value);
            }
        }

        /// <summary>
        /// Gets the <see cref="GWN.Contrib.Unity.Menu.MenuElement"/> with the specified name.
        /// </summary>
        /// <value></value>
        new public MenuElement this[string Name]
        {
            get
            {
                return (MenuElement)BaseGet(Name);
            }
        }

        /// <summary>
        /// Indexes the of.
        /// </summary>
        /// <param name="menu">The menu.</param>
        /// <returns></returns>
        public int IndexOf(MenuElement menu)
        {
            return BaseIndexOf(menu);
        }

        /// <summary>
        /// Adds the specified menu.
        /// </summary>
        /// <param name="menu">The menu.</param>
        public void Add(MenuElement menu)
        {
            BaseAdd(menu);
            // Add custom code here.
        }

        /// <summary>
        /// Adds a configuration element to the <see cref="T:System.Configuration.ConfigurationElementCollection"/>.
        /// </summary>
        /// <param name="element">The <see cref="T:System.Configuration.ConfigurationElement"/> to add.</param>
        protected override void BaseAdd(ConfigurationElement element)
        {
            BaseAdd(element, false);
            // Add custom code here.
        }

        /// <summary>
        /// Removes the specified menu.
        /// </summary>
        /// <param name="menu">The menu.</param>
        public void Remove(MenuElement menu)
        {
            if (BaseIndexOf(menu) >= 0)
                BaseRemove(menu.LinkText);
        }

        /// <summary>
        /// Removes at.
        /// </summary>
        /// <param name="index">The index.</param>
        public void RemoveAt(int index)
        {
            BaseRemoveAt(index);
        }

        /// <summary>
        /// Removes the specified name.
        /// </summary>
        /// <param name="name">The name.</param>
        public void Remove(string name)
        {
            BaseRemove(name);
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            BaseClear();
            // Add custom code here.
        }

    }
}
