﻿using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.ObjectBuilder2;

namespace GWN.Contrib.Unity.Strategy
{
	/// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
	/// MEF Strategy for populating context.Existing with
	/// references of any modules instantiated by MEF
	/// </summary>
	public class MEFStrategy : BuilderStrategy
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context">The context.</param>
		public override void PreBuildUp(IBuilderContext context)
		{
			// If this is a class derived from ModuleBase then
			// MEF may have loaded it so we'll check to ensure 
			// Unity doesn't stomp on it
			if (context.BuildKey.Type.BaseType.Name.Equals("ModuleBase"))
			{
				// Policy that will handle the population of the 
				// context.Existing property
				IMEFStrategyPolicy policy = context.Policies
					.Get<IMEFStrategyPolicy>(new NamedTypeBuildKey(typeof(IModule), "Module"));

				// If policy is not null then process for MEF.  It
				// will set the context.Existing property if it is
				// a module that was processed by MEF
				if (policy != null)
					policy.PopulateExistingFromMEF(context);
			}
			// Create or Buildup depending on status of context.Existing
			base.PreBuildUp(context);
		}
	}
}
