﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.ObjectBuilder2;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;

namespace GWN.Contrib.Unity.Strategy
{
	/// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
	/// MEF strategy policy interface
	/// </summary>
	public interface IMEFStrategyPolicy : IBuilderPolicy
	{
		/// <summary>
		/// Locates the specified context.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="context">The context.</param>
		/// <returns></returns>
		bool PopulateExistingFromMEF(IBuilderContext context);
	}

	/// <summary>
	/// MEF strategy policy
	/// </summary>
	public class MEFStrategyPolicy : IMEFStrategyPolicy
	{
		private IUnityContainer container;
		
		/// <summary>
		/// Initializes a new instance of the <see cref="MEFStrategyPolicy"/> class.
		/// </summary>
		/// <param name="container">The container.</param>
		public MEFStrategyPolicy(IUnityContainer container) 
		{
			this.container = container;
		}

		/// <summary>
		/// Locates the current context module from a list 
		/// of modules handled by MEF - if it exist then it
		/// sets context.Existing which effectively prevents
		/// Unity from creating a new instance; instead it
		/// does a buildup of the provided one
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="context">The context.</param>
		/// <returns></returns>
		public bool PopulateExistingFromMEF(IBuilderContext context)
		{
			IMEFStrategyData data = container.Resolve<IMEFStrategyData>();
			context.Existing = data.ModuleList
				.FirstOrDefault<IModule>(
					m => m.GetType().FullName.Equals(context.BuildKey.Type.FullName	));

			return true;
		}
	}
}
