﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Library.Demo.Tests.Mocks;
using System.Web.Routing;
using System.Configuration;
using GWN.Contrib.MVC2.Extensions;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Menu;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using GWN.Contrib.Unity;
using GWN.Contrib.Unity.Extensions;

namespace Library.Demo.Tests
{
	/// <summary>
	/// Summary description for LibraryTest
	/// </summary>
	[TestClass]
	public class ConfigurationFixture
	{
		[Dependency]
		public ISystemConfiguration MockConfig { get; set; }

		MockGlobal global = null;

		public ConfigurationFixture(){}

		[TestInitialize]
		public void TestInit()
		{
			// Initialize objects each test
			global = new MockGlobal();
			RouteTable.Routes.Clear();
			global.Application_Start();
			global.ContainerInstance.BuildUp(this);
		}

		/// <summary>
		/// Determines whether this instance [can serialize and deserialize].
		/// </summary>
		[TestMethod]
		public void CanSerializeAndDeserialize()
		{
			// We'll use the ObjectExtension class in GWN.Contrib.Unity to serialize/deserialize

			string xml = ((object)MockConfig).SerializeObject();
			Assert.IsNotNull(xml, "Was expecting an xml string value!");

			ISystemConfiguration result = 
				((object)MockConfig).DeserializeObject<ISystemConfiguration>(xml);

			Assert.AreEqual(2, result.AppSettings.Count, "Was expecting two AppSettings");
			Assert.AreEqual(2, result.ConnectionStrings.Count, "was expecting two ConnectionStrings");
    	}



		[TestMethod]
		public void CanReadMenuConfiguration()
		{
			Assert.AreEqual(1, MockConfig.WPFMenus.Count, "was expecting one menus!");
			Assert.AreEqual(3, MockConfig.WebsiteMenus.Count, "was expecting three menus!");
		}

		[TestMethod]
		public void CanReadConnectionAppSettings()
		{
			Assert.AreEqual(2, MockConfig.AppSettings.Count, "was expecting two AppSettings!");
			Assert.AreEqual(2, MockConfig.ConnectionStrings.Count, "was expecting two ConnectionStrings!!");
		}

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}
		#region Additional test attributes
		//
		// You can use the following additional attributes as you write your tests:
		//
		// Use ClassInitialize to run code before running the first test in the class
		// [ClassInitialize()]
		// public static void MyClassInitialize(TestContext testContext) { }
		//
		// Use ClassCleanup to run code after all tests in a class have run
		// [ClassCleanup()]
		// public static void MyClassCleanup() { }
		//
		// Use TestInitialize to run code before running each test 
		// [TestInitialize()]
		// public void MyTestInitialize() { }
		//
		// Use TestCleanup to run code after each test has run
		// [TestCleanup()]
		// public void MyTestCleanup() { }
		//
		#endregion
	}
}
