﻿using System;
using GWN.Contrib.Unity.Configuration;
using GWN.Contrib.Unity.Extensions;
using Microsoft.Practices.Composite.Logging;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Logger;

namespace GWN.Contrib.Unity.ASPNet
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// http://blogs.msdn.com/mpuleio/archive/2008/07/17/proof-of-concept-a-simple-di-solution-for-asp-net-webforms.aspx
    /// </summary>
    public class GlobalBase : System.Web.HttpApplication
    {
        /// <summary>
        /// Handles the Start event of the Application control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Application_Start(object sender, EventArgs e)
        {
            IUnityContainer container = Application.GetContainer();
            container
                .RegisterType<ILoggerFacade, DefaultHttpLogger>()
                .RegisterType<ISystemConfiguration, DefaultSysConfig>(new ContainerControlledLifetimeManager());

            ISystemConfiguration config = container.Resolve<ISystemConfiguration>();

            OnApplicationStart();
            OnConfigureContainer(container);
        }

        /// <summary>
        /// Called when [application start].
        /// </summary>
        protected virtual void OnApplicationStart()
        {

        }

        /// <summary>
        /// Called when [configure container].
        /// </summary>
        /// <param name="container">The container.</param>
        protected virtual void OnConfigureContainer(IUnityContainer container)
        {
        }
    }
}
