﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Extensions;
using GWN.Contrib.Unity.Interfaces;
using System.Diagnostics;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

namespace GWN.Contrib.Unity.ASPNet
{

    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// 
    /// http://blogs.msdn.com/mpuleio/archive/2008/07/17/proof-of-concept-a-simple-di-solution-for-asp-net-webforms.aspx
    ///
    /// Add the following to system.web and system.webserver
    /// <system.web>
    ///  <httpModules>
    ///    <add name="UnityHttpModule" type="GWN.Contrib.Unity.ASPNet.UnityHttpModule"/>
    ///  </httpModules>
    ///  
    ///<system.webServer>
    ///   <validation validateIntegratedModeConfiguration="false"/>
    ///      <modules>
    ///        <remove name="UnityHttpModule"/>
    ///        <add name="UnityHttpModule" preCondition="managedHandler" type="GWN.Contrib.Unity.ASPNet.UnityHttpModule"/>
    ///      </modules>
    /// </summary>
    public class UnityHttpModule : IHttpModule
    {
        #region IHttpModule Members

        ///<summary>
        ///Initializes a module and prepares it to handle requests.
        ///</summary>
        ///
        ///<param name="context">An <see cref="T:System.Web.HttpApplication"></see> that provides access to the methods, properties, and events common to all application objects within an ASP.NET application </param>
        public void Init(HttpApplication context)
        {
            context.PreRequestHandlerExecute += OnPreRequestHandlerExecute;
        }

        ///<summary>
        ///Disposes of the resources (other than memory) used by the module that implements <see cref="T:System.Web.IHttpModule"></see>.
        ///</summary>
        public void Dispose()
        {
        }

        #endregion

        /// <summary>
        /// Called when [pre request handler execute].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnPreRequestHandlerExecute(object sender, EventArgs e)
        {
            IHttpHandler handler = HttpContext.Current.Handler;
            HttpContext.Current.Application.GetContainer().BuildUp(handler.GetType(), handler);

            // User Controls are ready to be built up after the page initialization is complete
            Page page = HttpContext.Current.Handler as Page;
            if (page != null)
            {
                page.InitComplete += OnPageInitComplete;
            }
        }

        /// <summary>
        /// Get the controls in the page's control tree excluding the page 
        /// itself.  DetailsView will lose functionality if it is yielded
        /// to the process so we'll bypass it.
        /// </summary>
        /// <param name="root">The root.</param>
        /// <returns></returns>
        private IEnumerable<Control> GetControlTree(Control root)
        {
            foreach (Control child in root.Controls)
            {
                if (child is DetailsView)
                    continue;

                yield return child;
                foreach (Control c in GetControlTree(child))
                {
                    if (child is DetailsView)
                        continue;
                    yield return c;
                }
            }
        }

        /// <summary>
        /// Build up each control in the page's control tree
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnPageInitComplete(object sender, EventArgs e)
        {
            Page page = (Page)sender;
            IUnityContainer container = HttpContext.Current.Application.GetContainer();

            // Build up the page
            container.BuildUp(page.GetType(), page);

            // Now build up its controls
            foreach (Control c in GetControlTree(page))
                container.BuildUp(c.GetType(), c);
        }

    }
}
